;;;; The file laml.scm is the very first laml file to load.
;;;; It contains a number of variable and functions which must be defined whenever LAML is used.
;;;; It also contains a number of top level commands that activate the LAML tools.
;;;; Some of the variables - the configuration variables - are defined via the configuration file in the laml-config directory.
;;;; This file loads the scheme-system specific compatibility file and
;;;; the general.scm file from the library.
;;;; It is assumed that the value of the variable laml-dir is the full path of the LAML directory.
;;;; laml-dir must be defined when this file is loaded, and it must be ending in a "/".


; The LAML library and programs are written by Kurt Normark, Aalborg University, Denmark.
; Copyright (C) 1999  Kurt Normark, normark@cs.auc.dk.
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


;;; The configuration section.
;;; The configuration section is meant to be addapted in each new LAML installation.
;;; This section contains a few fundamental variables. The variables are defined via
;;; the configuration file in the laml-config directory. 

; ---------------------------------------------------------------------------------------------------
; CONFIGURATION SECTION:

;; Which Scheme system to use. 
;; A symbol. Currently we support 'scm and 'mzscheme (and to some degree 'drscheme).
(define scheme-system 'mzscheme)

;; The platform on which LAML is in use.
(define laml-platform 'unix)

;; The operating system on which LAML is in use.
(define operating-system 'solaris-7)

;; The scheme library relative to laml-dir. A string.
;; A single directory name (without ending slash).
;; You can change this if you use an alternative or experimental LAML library.
(define laml-library "lib4")

;; A variable that refers to the version of LAML, bound at LAML installation time.
;; .returns A string that contains the version number and a short description.
(define laml-version "Version 16.20 full (September 11, 2001)
")



; Determines how the laml processing is initiated. A symbol.
; rich: Information is transferred from the context such that the Scheme system knows the 
; file name and the start-up directory.
; poor: No information is transferred from the context.
; We now always use the value rich - in reality this variable does not play a role any longer.
(define laml-activation 'rich)


; The default name of an LAML output file.
; In case laml-activation is 'rich we use the name of the laml file to determine the name of the html output file.
; In poor laml activation situations we use the value of the variable laml-default-output-file.
; A file name without initial path and without extension.
; This variable does not play a role any longer.
(define laml-default-output-file "default")

; The default directory of LAML output.
; In case laml-activation is 'rich we use the name of the startup directory 
; (as returned by (startup-directory scheme-system) to determine this directory.
; The variable laml-default-output-directory is only used in case of poor laml-activation.
; This variable does not play a role any longer.
(define laml-default-output-directory "")


; END CONFIGURATIONS.
; ---------------------------------------------------------------------------------------------------

;;; Other variables.
;;; This part of the laml.scm file contains some other fundamental functions and variables.


;; The machine on which I use LAML-based software.
;; This variable is not used any place in the LAML software, so *you* can forget about.
;; I use it in the setup files of the LENO and course-plan systems. The variable allows
;; me to find out on which machine I am running. Files, on which I rely, may be placed different
;; places on different machines.
;; Possible values: cs-unix, home-pc, thinkpad
(define computer-system 'cs-unix)

;; An alias of laml-dir.
;; For backward compatibility.
;; Some LAML applications change this variable to a more local directory.
(define software-directory laml-dir)

;; An alias of laml-library. 
;; For backward compatibility
(define scheme-library laml-library)

;; Full path to the scheme library.
;; Normally the value of this variable is derived from laml-dir and laml-library. 
;; Ends in a slash.
(define the-library (string-append laml-dir laml-library "/"))

;; A global variable which signals some kind of variations in the loading of an LAML style or library.
;; If no load variation is present, the value of the variable will be #f. The interpretation of a non-false value
;; is entirely up to a style or a library.
;; The value of this variable is assigned by the style and laml-style form based on the last optional parameter of style or laml-style
(define laml-load-variation #f)

; ---------------------------------------------------------------------------------------------------

;;; Optional parameters in LAML software.
;;; In LAML version 14 and earlier we have been rather sloppy with respect to the handling of optional
;;; parameters of Scheme functions. From version 15 we have introduced the following simple support
;;; of optional parameters. Given the funciton <kbd>(lambda (r1 r2 . optional-parameters) ...)</kbd> the function
;;; <kbd>optional-parameter</kbd> (see below) is able to extract optional parameter number n. Non-used optional parameter can
;;; either be passed as the #f value (false in Scheme) or not passed at all. 

;; Return element n of optional-parameter-list. The first element is number 1.
;; In Scheme the optional parameters are captured as a list after the required parameters: <kbd>(define f (x y . optional-parameter-list) ...)</kbd>.
;; Please notice that if you pass optional parameter number i, the optional parameter 1, 2, i-1 must be passed explicitly.
;; (This is a non-symmetric property of optional parameters - and a motivation to a better alternative, which is keyword parameters).
;; If no optional third parameter - default-value - is given to the function optional-parameter the value #f counts as a non-passed optional parameter.
;; It is assumed as a precondition that optional-parameter-list is a proper list.
;; .form (optional-parameter n optional-parameter-list [default-value])
(define (optional-parameter n optional-parameter-list . optional-default-value)
  (let ((optional-default-value-1 (if (null? optional-default-value) #f (car optional-default-value))))  ; the old fashioned vay of handling it...
   (if  (> n (length optional-parameter-list))
      optional-default-value-1
      (list-ref optional-parameter-list (- n 1))))) 

; ---------------------------------------------------------------------------------------------------

;;; Library, style and local dir loading.
;;; The functions in this section loads LAML libraries and LAML styles.

;; Load file from the library determined by the variables laml-dir and laml-library.
(define (lib-load file-name)
  (load (string-append  the-library file-name)))

;; Load file from the startup directory.
(define (local-load file-name)
  (load (string-append (startup-directory) file-name)))

;; Load an LAML style.
;; .form (style style-spec style-spec load-variation)
;; .parameter style-spec The name of the style to load. A style-spec is without extension. However, the style file must have the scm extension.
;; .parameter style-base Optional: The directory which contains the style. If style-base is given it must be a directory (a slash terminated string) from which to load your style. If style-base is omitted, the style is loaded from styles subdirectory of the LAML directory.
;; .parameter load-variation Optional: A load-variation assigned to the global LAML variable laml-load-variation.
;; .example  (style "simple" #f 'xyz-variation)
;; .internal-references "also relevant" "laml-load-variation" "laml-style"
(define (style style-spec . optional-parameters)
 (let ((original-load-variation laml-load-variation))
  (let ((style-base (optional-parameter 1 optional-parameters))
        (load-variation (optional-parameter 2 optional-parameters))
       )
   (set! laml-load-variation load-variation)
   (if style-base
       (load (string-append style-base style-spec ".scm"))
       (load (string-append software-directory "styles/" style-spec ".scm")))
   (set! laml-load-variation original-load-variation))))

;; An alias to the function style which loads a LAML style.
;; The function style is an HTML mirror function, and as such there is a name clash. Please use laml-style instead of style.
;; .form (laml-style style-spec style-spec load-variation)
;; .parameter style-spec The name of the style to load. A style-spec is without extension. However, the style file must have the scm extension.
;; .parameter style-base Optional: The directory which contains the style. If style-base is given it must be a directory (a slash terminated string) from which to load your style. If style-base is omitted, the style is loaded from styles subdirectory of the LAML directory.
;; .parameter load-variation Optional: A load-variation assigned to the global LAML variable laml-load-variation.
;; .example  (laml-style "simple" #f 'xyz-variation)
;; .internal-references "also relevant" "style" "laml-load-variation" 
(define laml-style style)

; ---------------------------------------------------------------------------------------------------

;;; LAML contextual information.
;;; The functions in this section deal with the necessary context information,
;;; which must be passed to Scheme when we use LAML.

;; If possible return the name of the LAML source file (without extension).
;; This is only possible if the information somehow is passed to the Scheme execuctable.
;; In cases where it is not possible to know the source file name, return #f.
;; Notice: The parameter is not used, and should be avoided.
;; In order to be backward compatible, however, we allow a dummy parameter. 
(define (source-filename-without-extension . unused-parameter)
  (let ((cmd-line (laml-canonical-command-line)))
    (if cmd-line (cadr cmd-line) #f)))

;; Return the directory in which the Scheme system is started up.
;; If this information is not present, return #f.
;; Notice: The parameter is not used, and should be avoided.
;; In order to be backward compatible, however, we allow a dummy parameter. 
(define (startup-directory . unused-parameter)
  (let ((cmd-line (laml-canonical-command-line)))
    (if cmd-line (caddr cmd-line) #f)))


;; Return the contextual command line information passed to LAML upon activation.
;; Returns a list of lenght three, or #f if no command line activation exists.
;; The first element must be the symbol laml.
;; Element number two must be the laml source file name (without extension and initial path).
;; Element number three must be a slash terminated directory in which the source file resides.
;; This function must be redefined in the scheme-system dependent compatibility file.
(define (laml-canonical-command-line)
  (error "laml-canonical-command-line is not defined in scheme-system dependent compatibility file"))

;; Fake the contextual startup parameters to a specific source file name and a specific startup directory.
;; Both of the parameters must be strings, or the boolean value #f (in case the informations are unknown).
;; This function is useful for programmatic startup of LAML.
;; This function must be redefined in the scheme-system dependent compatibility file
(define (fake-startup-parameters source-file startup-dir)
  (error "fake-startup-parameters is not defined in scheme-system dependent compatibility file"))


;; Set the LAML startup directory to dir. 
;; Dir can be a full path, "..", or a directory relative to the current laml startup directory.
;; This is specialized call to fake-startup-parameters with only directory information.
(define (set-laml-startup-directory dir)
 (let ((start-dir (startup-directory)))
  (let ((abs-dir
          (cond ((and (equal? ".." dir) start-dir (parent-directory start-dir)) (parent-directory start-dir))
                ((and (not (absolute-file-path? dir)) start-dir) (string-append start-dir (ensure-final-character dir #\/)))
                ((absolute-file-path? dir) (ensure-final-character dir #\/))
                (else (display-error (string-append "Use an absolute file path!!!"))))))
    (if (directory-exists? abs-dir)
        (begin 
          (fake-startup-parameters 
             (source-filename-without-extension) abs-dir)
          (display-message (string-append "Using LAML in directory: " abs-dir)))
        (display-message (string-append "Non-existing directory: " abs-dir))))))

;; Set the LAML startup directory to dir. 
;; Dir can be a full path, "..", or a directory relative to the current laml startup directory.
;; A convenient and easy to remember alias to set-laml-startup-directory.
(define (laml-cd dir)
  (set-laml-startup-directory dir))

;; Returns the working LAML directory.
;; Similar to the UNIX command pwd.
;; An alias of the function startup-directory.
(define (laml-pwd)
  (startup-directory))

;; Returns a list of files and directories of the LAML startup directory (the current directory).
;; Similar to the UNIX command ls
(define (laml-ls)
  (directory-list (startup-directory)))
  

;; Set the LAML source file name (without extension) to file.
;; This is specialized call to fake-startup-parameters with only source file information.
(define (set-laml-source-file file)
  (fake-startup-parameters 
    file (startup-directory)))

;; Return the full path to the current source file name using the extension ext.
;; This function can be used conveniently to name the typical file for LAML to HTML transformations.
(define (full-source-path-with-extension ext)
  (string-append 
    (startup-directory)
    (source-filename-without-extension)
    "." ext))

; ---------------------------------------------------------------------------------------------------

;;; Programmatic loading of laml files. 
;;; Loading an laml file involes the setting of two pieces of context: The name of
;;; of the source file and the startup directory. The function laml-load sets these
;;; information and loads a file.

;; Load the laml file in full-file-path after faking the start up parameters.
;; full-file-path must be the full path of a laml file, including the laml extension.
(define (laml-load full-file-path)
 (let ((original-filename-wihtout-extension (source-filename-without-extension))
       (original-startup-dir (startup-directory)))
  (let ((filename-wihtout-extension (file-name-proper full-file-path))
        (startup-dir (file-name-initial-path full-file-path)))
    (fake-startup-parameters filename-wihtout-extension startup-dir)
    (load full-file-path)
    ; restore originals:
    (if (and original-filename-wihtout-extension original-startup-dir)
        (fake-startup-parameters original-filename-wihtout-extension original-startup-dir))
  )))

;; Load and execute the LAML file on the file file-name (a string).
;; The parameter file-name can be a full path with or without the file extension '.laml'.
;; Alternatively, file-name can be a file name with or without the '.laml' extension, in which
;; case we load the file from the laml starup directory.
;; This function is a flexible and versatile alternative to laml-load.
(define (laml file-name)
 (let* ((init-path (file-name-initial-path file-name))
        (extension (file-name-extension file-name))
        (proper-name (file-name-proper file-name))

        (init-path-1 (if (empty-string? init-path) (startup-directory) init-path))
        (extension-1 (if (empty-string? extension) "laml" extension))
        (proper-name-1 proper-name))
 
   (if (and (empty-string? init-path) (not (startup-directory)))
       (error "Please use full file path or set the laml startup directory via set-laml-startup-directory"))

   (laml-load (string-append init-path-1 proper-name-1 "." extension-1))))

;;; Interactive tool activation.
;;; The procedures in this section activate laml tools. 
;;; It is recommended that you activate the commands from an interactive LAML (Scheme) prompt.
;;; From emacs issue the editor command <kbd> run-laml-interactively </kbd>.<p>
;;; All the commands below work relative to the LAML working directory, which is changed by the procedure
;;; <kbd> laml-cd </kbd>. Use the command <kbd> laml-pwd </kbd> to check the LAML working directory.

;; Make documentation from scheme-source-file.
;; This function is meant to be called from a Scheme interpreter. You must first set lam-dir appropriately,
;; load laml.scm from your laml directory, and finally set the start directory via the procedure laml-cd.
;; .form (schemedoc scheme-source-file [relative-destination-dir manual-title manual-author-info manual-abstract])
;; .parameter scheme-source-file The source file in the current directory to be processed.
;; .parameter relative-destination-dir The relative directory in which to place the resulting html file. The directory must exist on beforehand. The default is the current directory. 
;; .parameter manual-title The title of the manual. The default is 'Manual of scheme-source-file'.
;; .parameter manual-author-info A list of name and affiliation of the author. The default value is no author.
;; .parameter manual-abstract The abstract of the manual. The default value is the 'four semicolon' extract abstract from the initial portion of the source file.
(define (schemedoc scheme-source-file . optional-parameters)
 (let ((this-dir (startup-directory))
       (relative-destination-dir (optional-parameter 1 optional-parameters ""))
       (manual-title (optional-parameter 2 optional-parameters scheme-source-file))
       (manual-author-info (optional-parameter 3 optional-parameters '()))
       (manual-abstract (optional-parameter 4 optional-parameters ""))
      )
  (load (string-append software-directory "tools/schemedoc/schemedoc.scm"))
  (let* ((manual-destination-directory (string-append this-dir relative-destination-dir))
         (doc-list (extract-documentation-from-scheme-file (string-append this-dir scheme-source-file)))
         (manual-name (file-name-proper scheme-source-file))
        )

    (laml-style "manual/manual")
    (set! laml-manual-stuff #f)
    (display-message "The Schemedoc tool.")
    (set-manual-title manual-title) 
    (apply set-manual-author manual-author-info)
    (set-manual-abstract (if (not (empty-string? manual-abstract)) manual-abstract extracted-manual-abstract))
    (set-manual-name manual-name)
    (set! manual-index-width-list  (list 180 320 350))
    (set! end-remark "This documentation has been extracted automatically from the Scheme
                      source file by means of the Schemedoc tool")
    (make-manual doc-list manual-destination-directory 
                 (string-append this-dir scheme-source-file))
    (display-message 
      (string-append "DONE. The manual of " scheme-source-file " is in the file "
                     relative-destination-dir manual-name ".html. "))
    (display-message (string-append "The file "  manual-name ".lsp" " contains a useful internal format."))

    )))


;; Parse the XML file file-name (a file name with or without xml extension) using the XML parser for LAML.
;; Writes the parse tree on the optional out-file-name.
;; .form (xml-parse in-file-name [out-file-name])
;; .parameter in-file-name The name of an XML file, with or without the xml file extension.
;; .parameter out-file-name The name of the file on which the parse tree is written. Defaults to the proper name of the xml file with and added lsp extension.
(define (xml-parse in-file-name . optional-parameters)
 (let* ((this-dir (startup-directory))
        (proper-file-name (file-name-proper in-file-name))
        (ext (file-name-extension in-file-name))
        (out-file-name (optional-parameter 1 optional-parameters (string-append proper-file-name "." "lsp")))
        (in-path (string-append this-dir proper-file-name (if (empty-string? ext) "" (string-append "." ext))))
        (out-path (string-append this-dir out-file-name))
      )
  (load (string-append laml-dir "tools/xml-html-support/xml-support.scm"))
  (parse-xml-file in-path out-path)))

;; Parse the HTML file file-name (a file name with or without extension) using the XML-based HTML parser for LAML.
;; Writes the parse tree on the optional out-file-name.
;; .form (html-parse in-file-name [out-file-name])
;; .parameter in-file-name The name of an HTML file, with or without the html file extension.
;; .parameter out-file-name The name of the file on which the parse tree is written. Defaults to the proper name of the html file with and added lsp extension.
(define (html-parse in-file-name . optional-parameters)
 (let* ((this-dir (startup-directory))
        (proper-file-name (file-name-proper in-file-name))
        (ext (file-name-extension in-file-name)) 
        (out-file-name (optional-parameter 1 optional-parameters (string-append proper-file-name "." "lsp"))) 
        (in-path (string-append this-dir proper-file-name (if (empty-string? ext) "" (string-append "." ext))))
        (out-path (string-append this-dir out-file-name))    
      )
  (load (string-append laml-dir "tools/xml-html-support/html-support.scm"))
  (parse-html-file in-path out-path)))

;; Pretty prints the XML file or XML parse tree in in-file-name and place the
;; pretty printed result in out-file-name. 
;; The input is assumed to be a parse tree if and only if the extension is lsp.
;; A XML file is parsed before pretty printing via use of the simple and
;; non-complete, non-validating XML parser from the LAML software package.
;; The optional file out-file-name defaults to in-file-name. 
;; In this case the original input file is overwritten.
;; If you care for your input file, it is strongly recommended that your output file does not overwrite your input file!
;; .form (xml-pp in-file-name [out-file-name single-lining indentation max-width])
;; .parameter in-file-name The file to pretty print
;; .parameter out-file-name The file on which to write the pretty printed result. Default value in-file-name.
;; .parameter single-lining A boolean variable that controls the line breaking; False means break consistently all forms. Default #t.
;; .parameter indentation The increment of indentation. Default value 3.
;; .parameter max-width The preferred maximum line width in the pretty printed file. Default value 80.
(define (xml-pp in-file-name . optional-parameters)
 (let* ((out-file-name (optional-parameter 1 optional-parameters in-file-name))
        (single-lining (optional-parameter 2 optional-parameters #t))
        (indentation (optional-parameter 3 optional-parameters 3))
        (max-width (optional-parameter 4 optional-parameters 80))
        (this-dir (startup-directory))
        (proper-in-file-name (file-name-proper in-file-name))
        (ext (file-name-extension in-file-name))
        (in-file-path (string-append this-dir in-file-name))
        (out-file-path (string-append this-dir out-file-name))
       )
  (load (string-append laml-dir "tools/xml-html-support/xml-support.scm"))

  (set! use-single-lining single-lining)
  (set! indentation-delta indentation)
  (set! prefered-maximum-width max-width)

  (write-text-file
    (pretty-print-xml-parse-tree 
      (if (equal? ext "lsp") (file-read in-file-path) (parse-xml in-file-path)))
    out-file-path)))

;; Pretty prints the HTML file or HTML parse tree in in-file-name and place the
;; pretty printed result in out-file-name. 
;; The input is assumed to be a parse tree if and only if the extension is lsp.
;; A HTML file is parsed before pretty printing via use of the non-validating HTML parser from the LAML software package.
;; The optional file out-file-name defaults to in-file-name. 
;; In this case the original input file is overwritten.
;; If you care for your input file, it is strongly recommended that your output file does not overwrite your input file!
;; .form (html-pp in-file-name [out-file-name single-lining indentation max-width])
;; .parameter in-file-name The file to pretty print
;; .parameter out-file-name The file on which to write the pretty printed result. Default value in-file-name.
;; .parameter single-lining A boolean variable that controls the line breaking; False means break consistently all forms. Default #t.
;; .parameter indentation The increment of indentation. Default value 3.
;; .parameter max-width The preferred maximum line width in the pretty printed file. Default value 80.
(define (html-pp in-file-name . optional-parameters)
 (let* ((out-file-name (optional-parameter 1 optional-parameters in-file-name))
        (single-lining (optional-parameter 2 optional-parameters #t))
        (indentation (optional-parameter 3 optional-parameters 3))
        (max-width (optional-parameter 4 optional-parameters 80))
        (this-dir (startup-directory))
        (proper-in-file-name (file-name-proper in-file-name))
        (ext (file-name-extension in-file-name))
        (in-file-path (string-append this-dir in-file-name))
        (out-file-path (string-append this-dir out-file-name))
       )
  (load (string-append laml-dir "tools/xml-html-support/html-support.scm"))

  (set! use-single-lining single-lining)
  (set! indentation-delta indentation)
  (set! prefered-maximum-width max-width)

  (write-text-file
    (pretty-print-html-parse-tree 
      (if (equal? ext "lsp") (file-read in-file-path) (parse-html in-file-path)))
    out-file-path)))



;; Parse the bibtex file, file-name, which is a bibtex file name without the bibtex extension.
;; Put the parsed result in file-name.lsp. In addition, deliver the result in the variable parse-result.
;; Finally, present the parsed file as HTML in file-name.html.
(define (bibtex file-name)
 (let ((this-dir (startup-directory))
       (proper-file-name (file-name-proper file-name))
       (ext (file-name-extension file-name))       
      )
  (lib-load "collect-skip.scm")
  (lib-load "file-read.scm")
  (load (string-append laml-dir "tools/bibtex/bibtex.scm"))
  (lib-load "time.scm")
  (lib-load "color.scm")
  (lib-load "html4.0-loose/basis.scm")
  (lib-load "html4.0-loose/surface.scm")
  (lib-load "html4.0-loose/convenience.scm")

  (parse-bibtex-file (string-append this-dir proper-file-name))
  (set! parse-result (reverse parse-result))

  (write-text-file
    (page 
      "Bibtex"
      (present-bibtex-entries parse-result (p)))
    (string-append this-dir proper-file-name ".html"))

  (display-message (string-append "The HTML output is in the file " (string-append this-dir proper-file-name ".html")))))

;; Pretty prints the Scheme or Lisp file - including comments -  in in-file-name and write the result to out-file-name.
;; Conventional comments (prefixed with semicolon) are converted with the Schemedoc procedure
;; lexical-to-syntactical-comments! before the pretty printing. In case you 
;; don't care about comments, you should probably use lisp-pp instead.
;; The optional file out-file-name defaults to in-file-name. 
;; In this case the original input file is overwritten.
;; It is strongly recommended that your output file does not overwrite your input file!
;; This function assumes that the general LAML library is loaded in advance.
;; .form (scheme-pp in-file-name [out-file-name single-lining indentation max-width])
;; .parameter in-file-name The file to pretty print
;; .parameter out-file-name The file on which to write the pretty printed result. Default value in-file-name.
;; .parameter single-lining A boolean variable that controls the line breaking; False means break consistently all forms. Default #t.
;; .parameter indentation The increment of indentation. Default value 3.
;; .parameter max-width The preferred maximum line width in the pretty printed file. Default value 80.
;; .internal-references "similar function" "scheme-pp-simple" 
(define (scheme-pp in-file-name . optional-parameters)
 (let* ((out-file-name (optional-parameter 1 optional-parameters in-file-name))
        (single-lining (optional-parameter 2 optional-parameters #t))
        (indentation (optional-parameter 3 optional-parameters 3))
        (max-width (optional-parameter 4 optional-parameters 80))
        (this-dir (startup-directory))
        (proper-in-file-name (file-name-proper in-file-name))
        (ext (file-name-extension in-file-name))
        (in-file-path (string-append this-dir in-file-name))
        (out-file-path (string-append this-dir out-file-name))
        (in-file-path-temp (string-append this-dir proper-in-file-name "-" "temp!!!" "." ext))
       )
  (lib-load "file-read.scm")
  (load (string-append laml-dir "tools/schemedoc/schemedoc.scm"))
  (set! COMMENT-FORM-START (string-append "(comment!!! "))
  (lib-load "scheme-pretty-printing.scm") 
  (set! use-single-lining single-lining)
  (set! indentation-delta indentation)
  (set! prefered-maximum-width max-width)
  (lexical-to-syntactical-comments! in-file-path in-file-path-temp) 
  (pretty-print-lisp-file in-file-path-temp out-file-path)
  (delete-file in-file-path-temp)
 )
)

;; Pretty prints the Scheme or Lisp file - without comment preservation - 
;; in in-file-name and write the result to out-file-name.
;; The pretty printing is simple because the conventional semicolon comments are lost.
;; The similar function scheme-pp preserves the comments during pretty printing.
;; The optional file out-file-name defaults to in-file-name. 
;; In this case the original input file is overwritten.
;; It is strongly recommended that your output file does not overwrite your input file!
;; This function assumes that the general LAML library is loaded in advance.
;; .form (scheme-pp-simple in-file-name [out-file-name single-lining indentation max-width])
;; .parameter in-file-name The file to pretty print
;; .parameter out-file-name The file on which to write the pretty printed result. Default value in-file-name.
;; .parameter single-lining A boolean variable that controls the line breaking; False means break consistently all forms. Default #t.
;; .parameter indentation The increment of indentation. Default value 3.
;; .parameter max-width The preferred maximum line width in the pretty printed file. Default value 80.
;; .internal-references "similar function" "scheme-pp" 
(define (scheme-pp-simple in-file-name . optional-parameters)
 (let* ((out-file-name (optional-parameter 1 optional-parameters in-file-name))
        (single-lining (optional-parameter 2 optional-parameters #t))
        (indentation (optional-parameter 3 optional-parameters 3))
        (max-width (optional-parameter 4 optional-parameters 80))
        (this-dir (startup-directory))
        (proper-in-file-name (file-name-proper in-file-name))
        (ext (file-name-extension in-file-name))
        (in-file-path (string-append this-dir in-file-name))
        (out-file-path (string-append this-dir out-file-name))
       )
  (lib-load "file-read.scm")
  (lib-load "scheme-pretty-printing.scm") 
  (set! use-single-lining single-lining)
  (set! indentation-delta indentation)
  (set! prefered-maximum-width max-width)
  (pretty-print-lisp-file in-file-path out-file-path)
 )
)

;; Convert the HTML file on in-file-name to an LAML file on out-file-name.
;; The conversion is done by parsing in-file-name, transforming the parse tree to LAML,
;; and by pretty printing the resulting LAML program.
(define (html-to-laml in-file-name out-file-name)
 (let* ((this-dir (startup-directory))
        (in-file-path (string-append this-dir in-file-name))
        (out-file-path (string-append this-dir out-file-name))
       )
  (load (string-append laml-dir "tools/xml-html-support/html-support.scm"))
  (lib-load "scheme-pretty-printing.scm") 
  (let* ((html-parse-tree (parse-html in-file-path)))
    (parse-tree-to-laml html-parse-tree out-file-path)
    (pretty-print-lisp-file out-file-path))))




; ---------------------------------------------------------------------------------------------------
;;; Language settings.

;; A variable which determines which language to use in selected parts of the LAML software complex.
;; The value of the variale must be a symbol.
;; Currently we only support 'danish and 'english. 'Danish is the default value.
(define language-preference 'danish) ; default

;; Return either danish or english, depending on the value of the global variable language-preference.
(define (text-choice danish english)
  (cond ((equal? language-preference 'english) english)
        ((equal? language-preference 'danish) danish)
        (else (error "Text: Problems in chosing language. Only 'english and 'danish are supported"))))

; ---------------------------------------------------------------------------------------------------
;;; LAML home URL.

;; The URL prefix of the LAML software home page at Aalborg University's WWW server.
(define laml-absolute-url-prefix "http://www.cs.auc.dk/~normark/scheme/")

;; Return a relative or absolute url prefix to the LAML home directory.
;; If possible a relative URL prefix is returned.
;; The parameter extra-level is an extra level to add;
;; As an example, extra-level should be 1 in case HTML files are organized in a sub-directory.
;; Normally, extra-level is 0 (zero).
;; The parameter start-dir is optional. It defaults to the value of (startup-directory).
(define (laml-home-url-prefix extra-level . start-dir)
 (let ((start-dir-1 (if (null? start-dir) (startup-directory) (car start-dir))))
  (if start-dir-1
   (let ((dir-diff (directory-level-difference start-dir-1 laml-dir)))
    (cond ((and dir-diff (number? dir-diff) (>= dir-diff 0)) 
             (string-append (repeat-string "../" (+ dir-diff extra-level))))
          (else laml-absolute-url-prefix)))
    laml-absolute-url-prefix)))
    

; ---------------------------------------------------------------------------------------------------
;;; Cosmetic welcome, ending and copyright functions. 


;; Initiating welcome and info text for interactive LAM tools. 
;; As of now this is entirely cosmetic.
(define (laml-welcome)
  (let ((vers (read-text-file (string-append laml-dir "distribution-version"))))
    (display (string-append "Welcome to LAML " vers ".")) (newline)
    (display "(C) Kurt Normark, Aalborg University, Denmark.") (newline) ))

;; This function is inteded to end an LAML file.
;; As of now this is entirely cosmetic.
(define (end-laml)
  (display "End of LAML processing") (newline))

;; Return an HTML comment with a copyright notice. 
;; It is also possible to return an empty string.
;; The value of this function is used in the generic page function called page-1.
;; Redefine this function for personal customization.
(define (copyright-clause)
  (let ((copyright-message ""))
    (if (not (empty-string? copyright-message))
        (html-comment copyright-message)
        "")))

;; Return a credit message to Kurt Nrmark about system-dk (the Danish name) and system-eng (the English name).
(define (credits system-dk system-eng)
   (string-append 
     (text-choice 
         (con system-dk " er designet og programmeret af Kurt Nrmark (c), Aalborg Universitet, med brug af "
                       (a-tag "http://www.cs.auc.dk/~normark/laml/" (font-color black "LAML")) " teknologi.")
         (con system-eng " is designed and programmed by Kurt Nrmark (c), Aalborg University, Denmark using " 
                      (a-tag "http://www.cs.auc.dk/~normark/laml/" (font-color black "LAML")) " technology.")
                  )))



; ---------------------------------------------------------------------------------------------------
; Loads the scheme-system specific compatibility file:

(lib-load (string-append "compatibility/" (symbol->string laml-platform) "_" (symbol->string operating-system) "_" (symbol->string scheme-system) ".scm"))
(lib-load "general.scm")

; ---------------------------------------------------------------------------------------------------
; Load the user's laml init file, if specified in the configuration file.

(if (and (not (equal? "/user/normark/.laml" "false")) (file-exists? "/user/normark/.laml"))
    (load "/user/normark/.laml"))

